
# q-expansion of newform 56.8.a.b, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 56
weight = 8
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [46], [-160], [-343], [-6840], [-2900], [16566], [-6718], [-976], [-61662], [-69236], [-533062], [183158], [966864], [-190268], [-785010], [2893594], [-95896], [-991644], [1068160], [2523458], [285848], [7094938], [-252390], [-1824794], [-9618848], [-16775316], [-2724284], [5398250], [-25129562], [9641456], [-22016046], [51643714], [-55526818], [20437782], [70022296], [-80519860], [-14998072], [37176964], [-107405116], [113389884], [-132257532], [-3877528], [-147799314], [-195010826], [66803844], [-167507580], [-79848632], [12868338], [-286473684], [98531658], [257316216], [257738918], [304338206], [-213793294], [77525920], [503407684], [-443691728], [-81266626], [455683434], [-431324310], [-835265416], [-128957854], [-549014904], [347991942], [773188702], [-755430032], [369427758], [383442832], [-676699948], [-884640430], [273898680], [-718777624], [-557666762], [350444120], [2097675956], [-370722550], [2158866244], [199648270], [1950022174], [-645883146], [265901982], [-1874486760], [1267946062], [-2614086264], [-2214424372], [-1797374126], [-1143516986], [-330188580], [3571890992], [-2079362758], [-2368319604], [1505478304], [3419908044], [-1706426204], [2402570784], [-476642796], [3351418902], [3050606658], [3712906942], [1337843584], [6967156158], [2246472462], [-3522364938], [3332824744], [7439897826], [2941106310], [-5804811174], [7637049000], [943486970], [7515671296], [8822486330], [-402928938], [-3231781926], [3341620912], [-1711538002], [-6408997922], [9494489668], [-9512266566], [-9056165280], [2082271640], [10340021690], [-5873332988], [-3394047604], [-10555285806], [658559450], [11425691850], [-762102916], [10396837820], [-18930586504], [-1987445816], [-8470190648], [-17028280008], [8182361690], [289321278], [20924166046], [-12113860808], [556243274], [4667067972], [6083203094], [-8275844830], [-3652295154], [-16509931816], [17612653012], [-13847510920], [-24002656836], [-3922695076], [8870291102], [2651798494], [1975075384], [-15975704846], [-19343322870], [-27800975300], [24082632972], [31656443204], [5153759472], [-23905075240], [43631908662], [4675541986], [-9723495352], [-17288824560], [23144796250], [46263523632], [17964229114], [7070403002], [39105823244], [21391497312], [-42678642296]]
